import json
import pickle
import logging
import six
from ipalib import api, errors, _
from ipalib import Command, Str, Int
from ipalib.plugable import Registry
from . import baseuser
from .baseuser import (baseuser, baseuser_output_params, validate_nsaccountlock)
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPAddMember, LDAPRemoveMember)
from ipalib import ngettext
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
Audit policy
""")

register = Registry()

user_output_params = baseuser_output_params

def mod_succ_paud(user_paud):

    mod_paud = str(user_paud)
    x1 = mod_paud.find('x')
    x2 = mod_paud.find(':', x1)
    mod_paud = list(mod_paud)
    mod_paud = mod_paud[x1+1:x2]
    mod_paud = ''.join(mod_paud)
    mod_paud = str(mod_paud)

    return (mod_paud, x1, x2)

def mod_fail_paud(user_paud):

    mod_paud = str(user_paud)
    x1 = mod_paud.rfind('x')
    mod_paud = list(mod_paud)
    mod_paud = mod_paud[x1+1:]
    mod_paud = ''.join(mod_paud)
    mod_paud = str(mod_paud)
    x2 = 0

    return (mod_paud, x1, x2)

def paud_hex(sumpaud):

    paud = (hex(sumpaud))
    paud = list(paud)
    paud[:2] = []
    paud = ''.join(paud)

    return (paud)

def change_user(ldap, dn, opt, *keys, **options):

    assert isinstance(dn, DN)

    if options.get('user') is not None:
        for memberuser in list(options.get('user', [])):
            user = memberuser
            user_attrs = {}
            user_attrs = api.Command.user_show(user)['result']

    if options.get('group') is not None:
        for membergroup in list(options.get('group', [])):
            group = membergroup
            user_attrs = {}
            user_attrs = api.Command.group_show(group)['result']
    try:
        if 'x-ald-aud-mask' in user_attrs:
            user_paud = user_attrs['x-ald-aud-mask'][0]
        else:
            user_paud = '0x0:0x0'

        if opt.get('opt1') == 'succ':
           (paudlists, truncated) = ldap.find_entries(
            None, ['*'],
            DN(('cn', 'audmasksucc'), ('cn', 'audit-policies'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
            ldap.SCOPE_ONELEVEL
           )
        elif opt.get('opt1') == 'fail':
             (paudlists, truncated) = ldap.find_entries(
              None, ['*'],
              DN(('cn', 'audmaskfail'), ('cn', 'audit-policies'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
              ldap.SCOPE_ONELEVEL
             )

        paud_map = {}
        for paudlist in paudlists:
            paud_map[(paudlist['cn'][0])] = (paudlist['x-ald-aud-mask'][0])
            if opt.get('opt1') == 'succ':
                mod_paud = mod_succ_paud(user_paud)
            elif opt.get('opt1') == 'fail':
                mod_paud = mod_fail_paud(user_paud)

        sumpaud = int(mod_paud[0], 16)
        if opt.get('opt2') == 'add':
            for name in keys:
                sumpaud = sumpaud | int(paud_map.get(name), 16)
        elif opt.get('opt2') == 'del':
            for name in keys:
                sumpaud = sumpaud & ~int(paud_map.get(name), 16)

        paud = paud_hex(sumpaud)
        x1 = mod_paud[1]
        x2 = mod_paud[2]

        if opt.get('opt1') == 'succ':
            sum_mask = user_paud[:x1+1] + paud + user_paud[x2:]
        elif opt.get('opt1') == 'fail':
            sum_mask = user_paud[:x1+1] + paud

        user_attrs.update({'x-ald-aud-mask': sum_mask})

    except:
        logging.exception("paud_update:")

    try:
        logging.debug('PARSEC: sum_mask=%s', sum_mask)
        if sum_mask != "0x0:0x0":
            logging.debug('PARSEC: mod sum_mask=%s', sum_mask)
            kw = {'setattr': unicode('x-ald-aud-mask=%s' % sum_mask)}
        else:
            logging.debug('PARSEC: del sum_mask user_paud=%s', user_paud)
            kw = {'delattr': unicode('x-ald-aud-mask=%s' % user_paud)}

        if options.get('user') is not None:
            keys = [user]
            api.Command['user_mod'](keys[-1], **kw)
            cn = 'user:' + keys[-1]
            amasktype = 0
        else:
            keys = [group]
            api.Command['group_mod'](keys[-1], **kw)
            cn = 'group:' + keys[-1]
            amasktype = 1

        if opt.get('opt2') == 'add':
            attr = {'x-ald-aud-mask':sum_mask, 'x-ald-aud-type': amasktype}

            recfind = api.Command['auditpolicy_find'](cn)
            if recfind.get('result') == []:
                attrs_list = api.Command['auditpolicy_add'](cn, **attr)
            else:
                api.Command['auditpolicy_mod'](cn, **kw)

        elif opt.get('opt2') == 'del':
            if sum_mask != "0x0:0x0":
                api.Command['auditpolicy_mod'](cn, **kw)
            else:
                api.Command['auditpolicy_del'](cn)

    except:
      logging.exception("ldap_update:")

def change_device(ldap, dn, opt, *keys, **options):

    assert isinstance(dn, DN)
    logging.debug('PARSEC: change_device Enter --------------------- opt1=%s', opt.get('opt1'))

    if options.get('parsecdevice') is not None:
        logging.debug('PARSEC: change_device Step 1')
        if opt.get('opt1') == 'succ':
            attr = 'audmasksucc'
        elif opt.get('opt1') == 'fail':
            attr = 'audmaskfail'
        logging.debug('PARSEC: change_device Step 2 attr=%s', attr)
        (paudlists, truncated) = ldap.find_entries(
        None, ['*'],
        DN(('cn', attr), ('cn', 'audit-policies'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
        ldap.SCOPE_ONELEVEL
        )

        logging.debug('PARSEC: change_device Step 3')
        for memberdevice in list(options.get('parsecdevice', [])):
            device = memberdevice
            device_attrs = {}
            device_attrs = api.Command.parsecdevice_show(device)['result']
            logging.debug('PARSEC: change_device device=%s', device)

            try:
                if attr in device_attrs:
                    sumpaud = int( device_attrs[attr][0], 16)
                else:
                    sumpaud = 0
                logging.debug('PARSEC: sumpaud in=%s', sumpaud)

                paud_map = {}
                for paudlist in paudlists:
                    paud_map[(paudlist['cn'][0])] = (paudlist['x-ald-aud-mask'][0])

                if opt.get('opt2') == 'add':
                    for name in keys:
                        logging.debug('PARSEC: add name=%s -> %s -> %s', name, paud_map.get(name), int(paud_map.get(name), 16))
                        sumpaud = sumpaud | int(paud_map.get(name), 16)
                        logging.debug('PARSEC: sumpaud step=%s', sumpaud)
                elif opt.get('opt2') == 'del':
                    for name in keys:
                        logging.debug('PARSEC: del name=%s -> %s -> %s', name, paud_map.get(name), int(paud_map.get(name), 16))
                        sumpaud = sumpaud & ~int(paud_map.get(name), 16)
                        logging.debug('PARSEC: sumpaud step=%s', sumpaud)
                logging.debug('PARSEC: sumpaud out=%s', sumpaud)

                device_attrs.update({attr: sumpaud})

            except:
                logging.exception("paud_update:")

            try:
                kw = {'setattr': unicode('%s=%s' % (attr, hex(sumpaud)))}
                keys = [device]
                api.Command['parsecdevice_mod'](keys[-1], **kw)

            except:
                logging.exception("ldap_update:")
            logging.debug('PARSEC: change_device done')


def add_audmasksucc_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)

    opt = {'opt1': 'succ', 'opt2': 'add'}
    change_user(ldap, dn, opt, *keys, **options)

def mov_audmasksucc_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)

    opt = {'opt1': 'succ', 'opt2': 'del'}
    change_user(ldap, dn, opt, *keys, **options)

def add_audmaskfail_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)

    opt = {'opt1': 'fail', 'opt2': 'add'}
    change_user(ldap, dn, opt, *keys, **options)

def mov_audmaskfail_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)

    opt = {'opt1': 'fail', 'opt2': 'del'}
    change_user(ldap, dn, opt, *keys, **options)

def add_devaudmasksucc_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)
    logging.debug('PARSEC: add_devaudmasksucc_attrs')

    opt = {'opt1': 'succ', 'opt2': 'add'}
    change_device(ldap, dn, opt, *keys, **options)

def mov_devaudmasksucc_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)
    logging.debug('PARSEC: mov_devaudmasksucc_attrs')

    opt = {'opt1': 'succ', 'opt2': 'del'}
    change_device(ldap, dn, opt, *keys, **options)

def add_devaudmaskfail_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)
    logging.debug('PARSEC: add_devaudmaskfail_attrs')

    opt = {'opt1': 'fail', 'opt2': 'add'}
    change_device(ldap, dn, opt, *keys, **options)

def mov_devaudmaskfail_attrs(ldap, dn, *keys, **options):

    assert isinstance(dn, DN)
    logging.debug('PARSEC: mov_devaudmaskfail_attrs')

    opt = {'opt1': 'fail', 'opt2': 'del'}
    change_device(ldap, dn, opt, *keys, **options)


def get_dn(self, *keys, **kwargs):

    try:
        if keys[-1] is not None:
           return self.api.Object.user.get_dn(*keys, **kwargs)
    except:
          logging.exception("get_dn:")
    return DN(self.container_dn, api.env.basedn)

@register()
class auditpolicy(LDAPObject):
    """
    Domain audit settings
    Настройки аудита домена
    """
    container_dn = DN(('cn', 'audit-policies'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('auditpolicy')
    object_name_plural = _('auditpolicys')
    object_class = ['x-ald-audit-policy', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-aud-mask', 'x-ald-aud-type']
    search_attributes = ['cn', 'x-ald-aud-mask', 'x-ald-aud-type']

    label = _('Audit masks')
    label_singular = _('Audit masks')

    takes_params = (
        Str('cn',
            cli_name='amaskname',
            label=_('Mask name'),
            primary_key=True,
        ),
        Str('x-ald-aud-mask',
            cli_name='amaskrank',
            label=_('Mask rank'),
            doc=_('integer to order the audit masks'),
        ),
        Str('x-ald-aud-type',
            cli_name='amasktype',
            label=_('Mask type'),
            doc=_('integer to order the audit type'),
        ),
    )

@register()
class auditpolicy_add(LDAPCreate):
    __doc__ = _('Create new audit mask')
    NO_CLI = True # BT-34042
    msg_summary = _('Added audit mask"%(value)s"')

    internal_options = ['x-ald-aud-type']

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)

        return dn

    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        return dn

@register()
class auditpolicy_del(LDAPDelete):
    __doc__ = _('Delete audit mask.')
    NO_CLI = True # BT-34042
    msg_summary = _('Deleted audit mask"%(value)s"')
    def pre_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)

        return dn

    def post_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)

        return True

@register()
class auditpolicy_mod(LDAPUpdate):
    __doc__ = _('Modify audit mask.')
    NO_CLI = True # BT-34042

@register()
class auditpolicy_find(LDAPSearch):
    __doc__ = _('Search for audit mask.')
    NO_CLI = True # BT-34042

    msg_summary = ngettext(
        '%(count)d audit mask matched',
        '%(count)d audit mask matched', 0
    )

@register()
class auditpolicy_show(LDAPRetrieve):
    __doc__ = _('Display audit mask.')
    NO_CLI = True # BT-34042

@register()
class paudit(LDAPObject):
    """
    Domain audit settings
    Настройки аудита домена
    """
    container_dn = DN(('cn', 'audmask'),
                      ('cn', 'audit-policies'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('paudit')
    object_name_plural = _('paudits')
    object_class = ['x-ald-audit-policy', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-aud-mask']
    search_attributes = ['cn', 'x-ald-aud-mask']

    label = _('Audit masks')
    label_singular = _('Audit masks')

    takes_params = (
        Str('cn',
            cli_name='amaskname',
            label=_('Mask name'),
            primary_key=True,
        ),
        Str('x-ald-aud-mask',
            cli_name='amaskrank',
            label=_('Mask rank'),
            doc=_('integer to order the audit masks'),
        ),
    )
 
def _existing_masks():
    """Формирует словарь всех имеющихся в БД масок событий аудита и их названий """
    mask_names = {}
    try:
        r = api.Command['paudit_find']() 
    except:
        logging.error("get_masknames: error of padit_find")
    lst = r.get('result', [])
    for dct in lst:
        maskname = dct.get('cn',[])[0]
        maskvalue = int(dct.get('x-ald-aud-mask',[])[0], 16)
        mask_names[maskvalue] = maskname
    if mask_names == {}:
        logging.error("get_masknames: masknames dict is empty")
    return mask_names

def _count_bits(number):
    """Подсчитывает количество единичных бит в числе"""
    if number == 0:
        logging.error("_count_bits: number is a zero value")
    length = number.bit_length()
    result = 0
    for i in range(length):
        result += number & 1
        number >>= 1
    return result

def _elementary_masks(exist_masks):
    """Формирует словарь имеющихся в БД простых (единичных)  масок событий аудита"""
    result = {}
    for mask, mask_name in exist_masks.items():
        if _count_bits(mask) == 1:
            result[mask_name] = mask
    if result == {}:
        logging.info("_elementary_masks: elementary masks dict is empty")
    return result
    
def _check_mask(mask_name, mask=None):
    exist_masks = _existing_masks()
    element_masks = _elementary_masks(exist_masks)
    if element_masks.get(mask_name, False):
        raise errors.ValidationError(message=_("Given mask name '%s' is elementary and cannot be added, removed or modified") % (mask_name))

    if mask is not None: # Только для вызова из paudit_add() и paudit_mod()
        try:
            mask = int(mask, 16)
        except ValueError:
            raise errors.ValidationError(message=_("Given mask value '%s' is invalid") % (mask))

        for item in exist_masks.keys():
            if item == mask:
                maskname = exist_masks[mask]
                raise errors.ValidationError(message=_("Given mask value %s already exists as '%s'") % (hex(mask), maskname))
        sum_elementary_masks = 0
        for elementary_mask_value in element_masks.values():
            sum_elementary_masks |= elementary_mask_value
        if mask & ~sum_elementary_masks or mask == 0: 
            raise errors.ValidationError(message=_("Given mask value %s does not match preset elementary masks") % (hex(mask)))

@register()
class paudit_add(LDAPCreate):
    __doc__ = _('Create new audit mask')

    msg_summary = _('Added audit mask"%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):

        assert isinstance(dn, DN)
        
        if options.get('x-ald-aud-mask', False):
            _check_mask(keys[-1], options['x-ald-aud-mask'])

        try:
           api.Command['audmasksucc_add'](keys[-1], **options)
           api.Command['audmaskfail_add'](keys[-1], **options)
        except:
          logging.exception("paud_update:")

        return dn

    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        return dn

@register()
class paudit_del(LDAPDelete):
    __doc__ = _('Delete audit mask.')

    msg_summary = _('Deleted audit mask"%(value)s"')

    def pre_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)
        _check_mask(keys[-1])
        return dn

    def post_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)
        try:
           api.Command['audmasksucc_del'](keys[-1])
           api.Command['audmaskfail_del'](keys[-1])
        except errors.NotFound:
            pass

        return True


@register()
class paudit_mod(LDAPUpdate):
    __doc__ = _('Modify audit mask.')

    msg_summary = _('Modified audit mask"%(value)s"')
    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        if options.get('x-ald-aud-mask', False):
            _check_mask(keys[-1], options['x-ald-aud-mask'])

        return dn

    def post_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)

        cat = options['x-ald-aud-mask']

        kw = {'setattr': unicode('x-ald-aud-mask=%s' % cat)}

        api.Command['audmasksucc_mod'](attrs_list, **kw)
        api.Command['audmaskfail_mod'](attrs_list, **kw)

        return dn


@register()
class paudit_find(LDAPSearch):
    __doc__ = _('Search for audit mask.')

    msg_summary = ngettext(
        '%(count)d audit mask matched',
        '%(count)d audit mask matched', 0
    )

@register()
class paudit_show(LDAPRetrieve):
    __doc__ = _('Display audit mask.')

@register()
class audmasksucc(LDAPObject):
    """
    Success Audit.
    """
    container_dn = DN(('cn', 'audmasksucc'),
                      ('cn', 'audit-policies'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('audmasksucc')
    object_name_plural = _('audmasksuccs')
    object_class = ['x-ald-audit-policy', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-aud-mask', 'x-ald-aud-type', 'member']
    search_attributes = ['cn', 'x-ald-aud-mask', 'x-ald-aud-type', 'member']

    attribute_members = {
        'member': ['user', 'group', 'parsecdevice'],
    }

    label = _('Success audit mask')
    label_singular = _('Success audit masks')

    takes_params = (
        Str('cn',
            cli_name='audmasksucc',
            label=_('Success audit name'),
            primary_key=True,
        ),
        Str('x-ald-aud-mask',
            cli_name='pauditid',
            label=_('Success audit rank'),
            doc=_('integer to order the audit masks'),
        ),
        Str('uid?',
            cli_name='user',
            label=_('User name'),
            doc=_('Manage mask of audit for specific user'),
        ),
        Str('memberuser_user?',
            label=_('Users'),
        ),
    )

@register()
class audmasksucc_add(LDAPCreate):
    __doc__ = _('Create new mask of Success Audit')
    NO_CLI = True # BT-34042
    msg_summary = _('Added mask of Success Audit"%(value)s"')

@register()
class audmasksucc_del(LDAPDelete):
    __doc__ = _('Delete mask of Success Audit.')
    NO_CLI = True # BT-34042
    msg_summary = _('Deleted mask of Success Audit"%(value)s"')


@register()
class audmasksucc_mod(LDAPUpdate):
    __doc__ = _('Modify mask of Success Audit.')
    NO_CLI = True # BT-34042
    msg_summary = _('Modified mask of Success Audit"%(value)s"')

@register()
class audmasksucc_find(LDAPSearch):
    __doc__ = _('Search for mask of Success Audit.')
    NO_CLI = True # BT-34042
    msg_summary = ngettext(
        '%(count)d mask of Success Audit matched',
        '%(count)d audit mask min matched', 0
    )

@register()
class audmasksucc_show(LDAPRetrieve):
    __doc__ = _('Display mask of Success Audit.')
    NO_CLI = True # BT-34042

@register()
class audmasksucc_add_audmasksucc(LDAPAddMember):
    __doc__ = _('Modify user min categories.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
        assert isinstance(dn, DN)

        for memberof in list(options.get('user', [])):

           user = memberof

           user_attrs = {}
           user_attrs = api.Command.user_show(user)['result']

           if not 'x-ald-aud-mask' in user_attrs:
             try:

                keys = [user]
                kw = {'setattr': unicode('x-ald-aud-mask=%s' % '0x0:0x0')}
                api.Command['user_mod'](keys[-1], **kw)

             except:
               logging.exception("ldap_update:")

        return dn

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)
        try:
            add_audmasksucc_attrs(ldap, dn, *keys, **options)
        except:
            logging.exception("files_write:")

        return (completed, dn)

@register()
class audmasksucc_remove_audmasksucc(LDAPRemoveMember):
    __doc__ = _('Remove users Success Audits.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        mov_audmasksucc_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class audmasksucc_add_devaudmasksucc(LDAPAddMember):

    __doc__ = _('Modify device  success audit.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
        assert isinstance(dn, DN)

        for memberof in list(options.get('parsecdevice', [])):

           device = memberof
           device_attrs = {}
           device_attrs = api.Command.parsecdevice_show(device)['result']

           if not 'audmasksucc' in device_attrs:
             try:
                keys = [device]
                kw = {'setattr': unicode('audmasksucc=%s' % 0)}
                api.Command['parsecdevice_mod'](keys[-1], **kw)

             except:
               logging.exception("ldap_update:")

        return dn

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)
        try:
            add_devaudmasksucc_attrs(ldap, dn, *keys, **options)
        except:
            logging.exception("files_write:")

        return (completed, dn)

@register()
class audmasksucc_remove_devaudmasksucc(LDAPRemoveMember):
    __doc__ = _('Remove device success audits.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        mov_devaudmasksucc_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class audmaskfail(LDAPObject):
    """
    Failure Audit.
    """
    container_dn = DN(('cn', 'audmaskfail'),
                      ('cn', 'audit-policies'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('audmaskfail')
    object_name_plural = _('audmaskfails')
    object_class = ['x-ald-audit-policy', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-aud-mask', 'x-ald-aud-type', 'member']
    search_attributes = ['cn', 'x-ald-aud-mask', 'x-ald-aud-type', 'member']

    attribute_members = {
        'member': ['user', 'group', 'parsecdevice'],
    }

    label = _('Failure audit mask')
    label_singular = _('Failure audit masks')

    takes_params = (
        Str('cn',
            cli_name='audmaskfail',
            label=_('Failure audit mask name'),
            primary_key=True,
        ),
        Str('x-ald-aud-mask',
            cli_name='paudmaskfail',
            label=_('Failure audit mask rank'),
            doc=_('integer to order the audit masks'),
        ),
        Str('uid?',
            cli_name='user',
            label=_('User name'),
            doc=_('Manage audit mask for specific user'),
        ),
        Str('memberuser_user?',
            label=_('Users'),
        ),
    )

@register()
class audmaskfail_add(LDAPCreate):
    __doc__ = _('Create new mask of Failure audit')
    NO_CLI = True # BT-34042
    msg_summary = _('Added mask of Failure audit"%(value)s"')

@register()
class audmaskfail_del(LDAPDelete):
    __doc__ = _('Delete mask of Failure audit.')
    NO_CLI = True # BT-34042
    msg_summary = _('Deleted mask of Failure audit"%(value)s"')

@register()
class audmaskfail_mod(LDAPUpdate):
    __doc__ = _('Modify mask of Failure audit.')
    NO_CLI = True # BT-34042
    msg_summary = _('Modified mask of Failure audit"%(value)s"')

@register()
class audmaskfail_find(LDAPSearch):
    __doc__ = _('Search for mask of Failure audit.')
    NO_CLI = True # BT-34042
    msg_summary = ngettext(
        '%(count)d mask of Failure audit matched',
        '%(count)d audit mask max matched', 0
    )

@register()
class audmaskfail_show(LDAPRetrieve):
    __doc__ = _('Display mask of Failure audit.')
    NO_CLI = True # BT-34042

@register()
class audmaskfail_add_audmaskfail(LDAPAddMember):
    __doc__ = _('Modify user masks.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
        assert isinstance(dn, DN)

        for memberof in list(options.get('user', [])):

           user = memberof

           user_attrs = {}
           user_attrs = api.Command.user_show(user)['result']

           if not 'x-ald-aud-mask' in user_attrs:
             try:

                keys = [user]
                kw = {'setattr': unicode('x-ald-aud-mask=%s' % '0x0:0x0')}
                api.Command['user_mod'](keys[-1], **kw)

             except:
               logging.exception("ldap_update:")

        return dn

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        add_audmaskfail_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class audmaskfail_remove_audmaskfail(LDAPRemoveMember):
    __doc__ = _('Remove users mask.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        mov_audmaskfail_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class audmaskfail_add_devaudmaskfail(LDAPAddMember):

    __doc__ = _('Modify device audit masks.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
        assert isinstance(dn, DN)

        for memberof in list(options.get('parsecdevice', [])):
           device = memberof
           device_attrs = {}
           device_attrs = api.Command.parsecdevice_show(device)['result']

           if not 'audmaskfail' in device_attrs:
             try:
                keys = [device]
                kw = {'setattr': unicode('audmaskfail=%s' % 0)}
                api.Command['parsecdevice_mod'](keys[-1], **kw)

             except:
               logging.exception("ldap_update:")

        return dn

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        add_devaudmaskfail_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class audmaskfail_remove_devaudmaskfail(LDAPRemoveMember):
    __doc__ = _('Remove device audit mask.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        mov_devaudmaskfail_attrs(ldap, dn, *keys, **options)

        return (completed, dn)
