import json
import logging
import six
from ipalib import api, errors, _
from ipalib import Command, Str, Int
from ipalib.plugable import Registry
from . import baseuser
from .baseuser import (baseuser, baseuser_output_params, validate_nsaccountlock)
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPAddMember, LDAPRemoveMember)
from ipalib import ngettext
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
Parsec capabilities

Manage Parsec capabilities.

""")

register = Registry()

user_output_params = baseuser_output_params

#topic = 'pcaps'

def add_caps_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    if options.get('user') is not None:
        try:

           for memberuser in list(options.get('user', [])):
              user = memberuser

              user_attrs = {}
              user_attrs = api.Command.user_show(user)['result']

              user_caps = user_attrs['x-ald-user-caps'][0]

              (caplists, truncated) = ldap.find_entries(
                None, ['*'],
                DN(('cn', 'parsec-caps'), ('cn', 'caps'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
                ldap.SCOPE_ONELEVEL
              )
              caps_map = {}
              for caplist in caplists:
                  caps_map[(caplist['cn'][0])] = (caplist['x-ald-user-cap'][0])

              mod_caps = list(user_caps)
              mod_caps[:2] = []
              mod_caps[-4:] = []
              mod_caps = ''.join(mod_caps)
              mod_caps = str(mod_caps)

              sumcaps = int(mod_caps, 16)
              for name in keys:
                 sumcaps = sumcaps | int(caps_map.get(name), 16)

                 caps = (hex(sumcaps))
                 caps = list(caps)
                 caps[:2] = []
                 caps = ''.join(caps)

              user_caps = user_caps[:2] + caps + user_caps[-4:]
              user_attrs.update({'x-ald-user-caps': user_caps})

        except:
          logging.exception("caps_update:")

        try:

           keys = [user]
           kw = {'setattr': unicode('x-ald-user-caps=%s' % user_caps)}
           api.Command['user_mod'](keys[-1], **kw)

        except:
          logging.exception("ldap_update:")

def mov_caps_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    if options.get('user') is not None:
        try:

           for memberof in list(options.get('user', [])):
              user = memberof

              user_attrs = {}
              user_attrs = api.Command.user_show(user)['result']

           if 'x-ald-user-caps' in user_attrs:
             user_caps = user_attrs['x-ald-user-caps'][0]

             (caplists, truncated) = ldap.find_entries(
               None, ['*'],
               DN(('cn', 'parsec-caps'), ('cn', 'caps'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
               ldap.SCOPE_ONELEVEL
             )
             caps_map = {}
             for caplist in caplists:
                 caps_map[(caplist['cn'][0])] = (caplist['x-ald-user-cap'][0])

             mod_caps = list(user_caps)
             mod_caps[:2] = []
             mod_caps[-4:] = []
             mod_caps = ''.join(mod_caps)
             mod_caps = str(mod_caps)

             sumcaps = int(mod_caps, 16)
             for name in keys:
                sumcaps = sumcaps & ~int(caps_map.get(name), 16)

                caps = (hex(sumcaps))
                caps = list(caps)
                caps[:2] = []
                caps = ''.join(caps)

             user_caps = user_caps[:2] + caps + user_caps[-4:]

             user_attrs.update({'x-ald-user-caps': user_caps})

        except:
          logging.exception("caps_update:")

        try:

           keys = [user]

           kw = {'setattr': unicode('x-ald-user-caps=%s' % user_caps)}

           api.Command['user_mod'](keys[-1], **kw)

        except:
          logging.exception("ldap_update:")


def get_dn(self, *keys, **kwargs):

    try:
        if keys[-1] is not None:
           return self.api.Object.user.get_dn(*keys, **kwargs)
    except:
          logging.exception("get_dn:")
    return DN(self.container_dn, api.env.basedn)


@register()
class pcaps(LDAPObject):
    """
    CAP Level
    """
    container_dn = DN(('cn', 'parsec-caps'),
                      ('cn', 'caps'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('pcaps')
    object_name_plural = _('pcaps_s')
    object_class = ['x-ald-user', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-user-cap', 'member']
    search_attributes = ['cn', 'x-ald-user-cap', 'member']

    attribute_members = {
        'member': ['user'],
    }

    label = _('CAP Levels')
    label_singular = _('CAP Level')

    takes_params = (
        Str('cn',
            cli_name='pcapname',
            label=_('Parsec CAP name'),
            primary_key=True,
        ),
        Int('x-ald-user-cap',
            cli_name='pcapid',
            label=_('Parsec CAP rank'),
            doc=_('integer to order the Parsec CAP rank'),
        ),
        Str('uid?',
            cli_name='user',
            label=_('User name'),
            doc=_('Manage CAPs for specific user'),
        ),
        Str('memberuser_user?',
            label=_('Users'),
        ),
    )

@register()
class pcaps_add(LDAPCreate):
    __doc__ = _('Create new Parsec CAP')

    msg_summary = _('Added Parsec CAP"%(value)s"')


@register()
class pcaps_del(LDAPDelete):
    __doc__ = _('Delete Parsec CAP.')

    msg_summary = _('Deleted Parsec CAP"%(value)s"')


@register()
class pcaps_mod(LDAPUpdate):
    __doc__ = _('Modify Parsec CAP.')

    msg_summary = _('Modified Parsec CAP"%(value)s"')


@register()
class pcaps_find(LDAPSearch):
    __doc__ = _('Search for Parsec CAP.')

    msg_summary = ngettext(
        '%(count)d Parsec CAP matched',
        '%(count)d Parsec CAP matched', 0
    )


@register()
class pcaps_show(LDAPRetrieve):
    __doc__ = _('Display Parsec CAP.')


@register()
class pcaps_add_pcaps(LDAPAddMember):

    __doc__ = _('Modify user CAPs.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    logging.basicConfig(level=logging.DEBUG, filename='/tmp/myapp.log')

    def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
        assert isinstance(dn, DN)

        for memberof in list(options.get('user', [])):

           user = memberof

           user_attrs = {}
           user_attrs = api.Command.user_show(user)['result']

           if not 'x-ald-user-caps' in user_attrs:
             try:

                keys = [user]
                kw = {'setattr': unicode('x-ald-user-caps=%s' % '0x0:0x0')}
                api.Command['user_mod'](keys[-1], **kw)

             except:
               logging.exception("ldap_update:")

        return dn

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        add_caps_attrs(ldap, dn, *keys, **options)

        return (completed, dn)


@register()
class pcaps_remove_member(LDAPRemoveMember):
    __doc__ = _('Remove users CAPs.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    logging.basicConfig(level=logging.DEBUG, filename='/tmp/myapp.log')

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        mov_caps_attrs(ldap, dn, *keys, **options)

        return (completed, dn)
