# -*- coding: koi8-r -*-
import sys
import json
import logging
import six
from ipalib import api, errors, _
from ipalib import Command, Str, Int
from ipalib.plugable import Registry
from . import baseuser
from .baseuser import (baseuser, baseuser_output_params, validate_nsaccountlock)
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPAddMember, LDAPRemoveMember)
from ipalib import ngettext
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
Parsec categories

Manage Parsec confidential categories.

""")

register = Registry()

CATEGORIES_PATTERN='0x[1248]0*'
CATEGORIES_PATTERN_ERRMSG='may only include 0x +numbers 1,2,4,8 and 0.'

user_output_params = baseuser_output_params

def maccat_sync( ldap):
    logging.debug('PARSEC: maccat_sync: Enter')
    try:
        (basecats, truncated) = ldap.find_entries(
        '(cn=*)', ['cn', 'x-ald-mac-category'],
        DN(('cn', 'mac-categories'), ('cn', 'mac-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
        ldap.SCOPE_ONELEVEL
        )
    except errors.NotFound:
        logging.debug('PARSEC: maccat_sync: base cats not found')

    for name in ( 'macmaxcat', 'macmincat', 'macdevcat'):
        try:
            (catlist, truncated) = ldap.find_entries(
            '(cn=*)', ['cn', 'x-ald-mac-category'],
            DN(('cn', name), ('cn', 'mac-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
            ldap.SCOPE_ONELEVEL
            )
            for cat in catlist:
                for basecat in basecats:
                    if cat['cn'] == basecat['cn']:
                        if not int(basecat['x-ald-mac-category'][0], 16) == int(cat['x-ald-mac-category'][0], 16):
                            kw = { 'x-ald-mac-category' : u"%s" % basecat['x-ald-mac-category'][0] }
                            logging.debug('PARSEC: maccat_sync_sync: cat %s update from %s to %s kw=%s', cat['cn'], cat['x-ald-mac-category'], basecat['x-ald-mac-category'], kw)
                            api.Command['%s_mod' % name](basecat['cn'][0], **kw)
                        break
                else:
                    logging.debug('PARSEC: maccat_sync_sync: cat %s to delete', cat['cn'][0])
                    api.Command['%s_del'%name](cat['cn'][0])

            for basecat in basecats:
    #            logging.debug('PARSEC: maccat_sync_sync: check basecat %s-%s', name, basecat['cn'])
                for cat in catlist:
                    if cat['cn'] == basecat['cn']:
                        break
                else:
                    kw = { 'x-ald-mac-category' : u"%s" % basecat['x-ald-mac-category'][0]}
                    logging.debug('PARSEC: maccat_sync_sync: cat %s to append kw=%s', basecat['cn'], kw)
                    api.Command['%s_add' % name](basecat['cn'][0], **kw)

        except errors.NotFound:
            logging.debug('PARSEC: maccat_sync: no %s found', name)

def mod_min_pcat(user_pcat):

    mod_pcat = str(user_pcat)
    x1 = mod_pcat.find('x')
    x2 = mod_pcat.find(':', x1)
    mod_pcat = list(mod_pcat)
    mod_pcat = mod_pcat[x1+1:x2]
    mod_pcat = ''.join(mod_pcat)
    mod_pcat = str(mod_pcat)

    return (mod_pcat, x1, x2)

def mod_max_pcat(user_pcat):

    mod_pcat = str(user_pcat)
    x1 = mod_pcat.rfind('x')
    mod_pcat = list(mod_pcat)
    mod_pcat = mod_pcat[x1+1:]
    mod_pcat = ''.join(mod_pcat)
    mod_pcat = str(mod_pcat)
    x2 = 0

    return (mod_pcat, x1, x2)

def pcat_hex(sumpcat):

    pcat = (hex(sumpcat))
    pcat = list(pcat)
    pcat[:2] = []
    pcat = ''.join(pcat)

    return pcat

def change_user(ldap, dn, opt, *keys, **options):

    assert isinstance(dn, DN)

    if options.get('user') is not None:
        try:
            for memberuser in list(options.get('user', [])):

                user = memberuser
                user_attrs = {}
                user_attrs = api.Command.user_show(user)['result']

                user_pcat = user_attrs['x-ald-user-mac'][0]

                if opt.get('opt1') == 'min':
                    (pcatlists, truncated) = ldap.find_entries(
                     None, ['*'],
                     DN(('cn', 'macmincat'), ('cn', 'mac-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
                     ldap.SCOPE_ONELEVEL
                    )
                elif opt.get('opt1') == 'max':
                    (pcatlists, truncated) = ldap.find_entries(
                     None, ['*'],
                     DN(('cn', 'macmaxcat'), ('cn', 'mac-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
                     ldap.SCOPE_ONELEVEL
                    )

                pcat_map = {}
                for pcatlist in pcatlists:
                    pcat_map[(pcatlist['cn'][0])] = (pcatlist['x-ald-mac-category'][0])

                    if opt.get('opt1') == 'min':
                        mod_pcat = mod_min_pcat(user_pcat)

                    elif opt.get('opt1') == 'max':
                        mod_pcat = mod_max_pcat(user_pcat)

                sumpcat = int(mod_pcat[0], 16)

                if opt.get('opt2') == 'add':
                    for name in keys:
                        sumpcat = sumpcat | int(pcat_map.get(name), 16)
                        #sumpcat = sumpcat + int(pcat_map.get(name))
                elif opt.get('opt2') == 'del':
                    for name in keys:
                        sumpcat = sumpcat & ~int(pcat_map.get(name), 16)
                        #sumpcat = sumpcat - int(pcat_map.get(name))

                pcat = pcat_hex(sumpcat)
                x1 = mod_pcat[1]
                x2 = mod_pcat[2]

                if opt.get('opt1') == 'min':
                    user_pcat = user_pcat[:x1+1] + pcat + user_pcat[x2:]
                elif opt.get('opt1') == 'max':
                    user_pcat = user_pcat[:x1+1] + pcat

                user_attrs.update({'x-ald-user-mac': user_pcat})

                keys = [user]
                kw = {'setattr': unicode('x-ald-user-mac=%s' % user_pcat)}

                api.Command['user_mod'](keys[-1], **kw)

        except:
            logging.exception("ldap_update:")

def change_device(ldap, dn, opt, *keys, **options):

    assert isinstance(dn, DN)

    logging.debug( 'PARSEC: Enter change device')
    if options.get('parsecdevice') is not None:
        try:
            for memberdevice in list(options.get('parsecdevice', [])):

                logging.debug( 'PARSEC: Device is %s', memberdevice)
                device = memberdevice
                device_attrs = {}
                device_attrs = api.Command.parsecdevice_show(device)['result']
#                logging.debug( 'PARSEC: Device attrs %s', device_attrs)

                device_lev = int( device_attrs['x-ald-mac-level'][0])
                device_cat = int( device_attrs['x-ald-mac-category'][0],16)
                logging.debug( 'PARSEC: Device lev %s cat %s', device_lev, device_cat)

                (pcatlists, truncated) = ldap.find_entries(
                 None, ['*'],
                 DN(('cn', 'macdevcat'), ('cn', 'mac-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
                 ldap.SCOPE_ONELEVEL
                )
#                logging.debug( 'PARSEC: pcat_list %s', pcatlists)

                pcat_map = {}
                for pcatlist in pcatlists:
                    pcat_map[(pcatlist['cn'][0])] = (pcatlist['x-ald-mac-category'][0])

                logging.debug( 'PARSEC: pcat_map %s', pcat_map)
                logging.debug( 'PARSEC: keys %s', keys)

                if opt.get('opt2') == 'add':
                    for name in keys:
                        device_cat = device_cat | int(pcat_map.get(name), 16)
                        logging.debug( 'PARSEC: add %s - %s cat %s', name, pcat_map.get(name), device_cat)

                elif opt.get('opt2') == 'del':
                    for name in keys:
                        device_cat = device_cat & ~int(pcat_map.get(name), 16)

                logging.debug( 'PARSEC: lev %s cat %s', device_lev, device_cat)
#               device_mac='%s:%s'%(device_lev, hex(device_cat))
#                logging.debug( 'PARSEC: mac %s', device_mac)
#                device_attrs.update({'x-ald-devac-mac': device_mac})
                device_attrs.update({'x-ald-mac-category': device_cat})

                keys = [device]
#                kw = {'setattr': unicode('x-ald-devac-mac=%s' % device_mac)}
#                api.Command['parsecdevice_mod'](keys[-1], **kw)
                kw = {'setattr': unicode('x-ald-mac-category=%s' % hex(device_cat))}
                api.Command['parsecdevice_mod'](keys[-1], **kw)

                logging.debug( 'PARSEC: done')

        except:
            logging.exception("ldap_update:")

def recount_users_pcat(ldap, dn, attrs_list, oldpcatid, newpcatid, *keys):

    try:
        list_users_pcatmin = api.Command.user_find(in_macmincat=attrs_list)
        list_users_pcatmin = list_users_pcatmin['result']

        memberusers = [entry['uid'][0] for entry in list_users_pcatmin]

        for user in memberusers:

            user_attrs = {}
            user_attrs = api.Command.user_show(user)['result']
            user_pcat = user_attrs['x-ald-user-mac'][0]

            old_user_pcat = mod_min_pcat(user_pcat)

            recalc_user_pcat = int(old_user_pcat[0], 16) & ~int(oldpcatid, 16)
            #recalc_user_pcat = int(old_user_pcat[0], 16) - int(oldpcatid)
            recalc_user_pcat = pcat_hex(recalc_user_pcat)

            sumpcat = int(str(recalc_user_pcat), 16) | int(str(newpcatid), 16)

            recalc_user_pcat = pcat_hex(sumpcat)

            x1 = old_user_pcat[1]
            x2 = old_user_pcat[2]

            user_pcat = user_pcat[:x1+1] + str(recalc_user_pcat) + user_pcat[x2:]
            user_attrs.update({'x-ald-user-mac': user_pcat})

            keys = [user]
            kw = {'setattr': unicode('x-ald-user-mac=%s' % user_pcat)}

            api.Command['user_mod'](keys[-1], **kw)

    except:
          logging.exception("find_users_in_pcatmin:")
    try:
        list_users_pcatmax = api.Command.user_find(in_macmaxcat=attrs_list)
        list_users_pcatmax = list_users_pcatmax['result']

        memberusers = [entry['uid'][0] for entry in list_users_pcatmax]

        for user in memberusers:

            user_attrs = {}
            user_attrs = api.Command.user_show(user)['result']
            user_pcat = user_attrs['x-ald-user-mac'][0]

            old_user_pcat = mod_max_pcat(user_pcat)

            recalc_user_pcat = int(old_user_pcat[0], 16) & ~int(oldpcatid, 16)
            #recalc_user_pcat = int(old_user_pcat[0], 16) - int(oldpcatid)
            recalc_user_pcat = pcat_hex(recalc_user_pcat)

            sumpcat = int(str(recalc_user_pcat), 16) | int(str(newpcatid), 16)

            recalc_user_pcat = pcat_hex(sumpcat)

            x1 = old_user_pcat[1]

            user_pcat = user_pcat[:x1+1] + str(recalc_user_pcat)
            user_attrs.update({'x-ald-user-mac': user_pcat})

            keys = [user]
            kw = {'setattr': unicode('x-ald-user-mac=%s' % user_pcat)}

            api.Command['user_mod'](keys[-1], **kw)
    except:
          logging.exception("find_users_in_pcatmax:")

def add_macmincat_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    opt = {'opt1': 'min', 'opt2': 'add'}
    change_user(ldap, dn, opt, *keys, **options)

def mov_macmincat_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    opt = {'opt1': 'min', 'opt2': 'del'}
    change_user(ldap, dn, opt, *keys, **options)

def add_macmaxcat_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    opt = {'opt1': 'max', 'opt2': 'add'}
    change_user(ldap, dn, opt, *keys, **options)

def add_devicecat_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    opt = {'opt1': 'max', 'opt2': 'add'}
    change_device(ldap, dn, opt, *keys, **options)

def mov_pcat_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    opt = {'opt1': 'max', 'opt2': 'del'}
    change_user(ldap, dn, opt, *keys, **options)

def mov_device_attrs(ldap, dn, *keys, **options):
    assert isinstance(dn, DN)

    opt = {'opt1': 'max', 'opt2': 'del'}
    change_device(ldap, dn, opt, *keys, **options)

def get_dn(self, *keys, **kwargs):

    try:
        if keys[-1] is not None:
           return self.api.Object.user.get_dn(*keys, **kwargs)
    except:
          logging.exception("get_dn:")
    return DN(self.container_dn, api.env.basedn)

@register()
class pcategories(LDAPObject):
    """
    Levels min categories
    """
    container_dn = DN(('cn', 'mac-categories'),
                      ('cn', 'mac-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('pcategories')
    object_name_plural = _('pcategoriess')
    object_class = ['x-ald-mac-category-object', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-mac-category']
    search_attributes = ['cn', 'x-ald-mac-category']

    label = _('Category levels')
    label_singular = _('Category levels')

    takes_params = (
        Str('cn',
            cli_name='pcatname',
            label=_('Category name'),
            primary_key=True,
        ),
        Str('x-ald-mac-category',
            pattern=CATEGORIES_PATTERN,
            pattern_errmsg=CATEGORIES_PATTERN_ERRMSG,
            cli_name='pcatid',
            label=_('Category rank'),
            doc=_('hex to order the MAC categories'),
        ),
    )
    allow_rename = True

@register()
class pcategories_add(LDAPCreate):
    __doc__ = _('Create new MAC category')

    msg_summary = _('Added MAC category"%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)
        logging.debug('PARSEC: pcategories_add');

        try:
            #dec = int(options.get("x-ald-mac-category"), 16)
            #options.update({"x-ald-mac-category": (dec)})
            #entry_attrs['x-ald-mac-category'] = dec
            api.Command['macmincat_add'](keys[-1], **options)
            api.Command['macmaxcat_add'](keys[-1], **options)
            api.Command['macdevcat_add'](keys[-1], **options)
        except:
            logging.exception("pcat_update:")

        return dn

def maccat_check( ldap, dn, keys):
    logging.debug('PARSEC: pcategories_del key=%s, dn=%s', keys, dn)
    str = ''
    for key in keys:
        logging.debug('PARSEC: pcategories_del key=%s, dn=%s', key, dn)
        for name in ( 'macmaxcat', 'macmincat'):
            catdn = DN(('cn', key), ('cn', name), ('cn', 'mac-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn)
            logging.debug('PARSEC: pcategories_del catdn=%s', catdn)
            try:
                (users, truncated) = ldap.find_entries( '(memberof=%s)' % (catdn), ['uid'], DN(('cn', 'users'), ('cn', 'accounts'), api.env.basedn), ldap.SCOPE_ONELEVEL)
                for user in users:
                    if len(str) > 0:
                        str = str + ', '
                    str = str + '"' + user['uid'][0] + '"';
                logging.debug('PARSEC: pcategories_del users=%s', str)
            except errors.NotFound:
                logging.debug('PARSEC:pcategories_del cat %s has no dependent users', key)
        catdn = DN(('cn', key), ('cn', 'macdevcat'), ('cn', 'mac-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn)
        try:
            (devices, truncated) = ldap.find_entries( '(memberof=%s)' % (catdn), ['cn'], DN(('cn', 'devices'), ('cn', 'dev-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn), ldap.SCOPE_ONELEVEL)
            for dev in devices:
                if len(str) > 0:
                    str = str + ', '
                str = str + '"' + dev['cn'][0] + '"'
            logging.debug('PARSEC: pcategories_del devices=%s', str)
        except errors.NotFound:
            logging.debug('PARSEC: pcategories_del cat %s has no dependent devices', key)
        if len(str) > 0:
            raise errors.DependentEntry(label=u'users or devices', key=key, dependent=str)
            return None

    return dn

@register()
class pcategories_del(LDAPDelete):
    __doc__ = _('Delete MAC category.')

    msg_summary = _('Deleted MAC category"%(value)s"')

    def pre_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)
        return maccat_check( ldap, dn, keys)

    def post_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)

        try:
           api.Command['macmincat_del'](keys[-1])
           api.Command['macmaxcat_del'](keys[-1])
           api.Command['macdevcat_del'](keys[-1])
        except errors.NotFound:
            pass

        return True

@register()
class pcategories_mod(LDAPUpdate):
    __doc__ = _('Modify MAC category.')

    msg_summary = _('Modified MAC category"%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)

        if maccat_check( ldap, dn, keys) is None:
            return None

        try:
            get_oldpcatid = api.Command['pcategories_find'](keys[0])['result'][0]
            self.oldpcatid = get_oldpcatid['x-ald-mac-category'][0]
        except:
          logging.exception("get_oldpcatid_id:")

        return dn

    def post_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
        assert isinstance(dn, DN)

        newpcatid = options['x-ald-mac-category']
        oldpcatid = self.oldpcatid
        kw = {'setattr': unicode('x-ald-mac-category=%s' % str(newpcatid))}

        api.Command['macmincat_mod'](attrs_list, **kw)
        api.Command['macmaxcat_mod'](attrs_list, **kw)
        api.Command['macdevcat_mod'](attrs_list, **kw)
        recount_users_pcat(ldap, dn, attrs_list, oldpcatid, newpcatid, *keys)

        return dn

@register()
class pcategories_find(LDAPSearch):
    __doc__ = _('Search for MAC category.')

    msg_summary = ngettext(
        '%(count)d MAC category matched',
        '%(count)d MAC category matched', 0
    )

    def pre_callback(self, ldap, filters, attrs_list,
                     base_dn, scope, *args, **options):
        assert isinstance(base_dn, DN)
        maccat_sync( ldap)
        return (filters, base_dn, scope)

@register()
class pcategories_show(LDAPRetrieve):
    __doc__ = _('Display MAC category.')

#topic = 'macmincat'

@register()
class macmincat(LDAPObject):
    """
    Levels min categories
    """
    container_dn = DN(('cn', 'macmincat'),
                      ('cn', 'mac-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('macmincat')
    object_name_plural = _('macmincats')
    object_class = ['x-ald-mac-category-object', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-mac-category', 'member']
    search_attributes = ['cn', 'x-ald-mac-category', 'member']

    attribute_members = {
        'member': ['user'],
    }

    label = _('Min category levels')
    label_singular = _('Min category levels')

    takes_params = (
        Str('cn',
            cli_name='macmincat',
            label=_('Min category name'),
            primary_key=True,
        ),
        Str('x-ald-mac-category',
            cli_name='pcategoryid',
            label=_('Min category rank'),
            doc=_('integer to order the MAC categories'),
        ),
        Str('uid?',
            cli_name='user',
            label=_('User name'),
            doc=_('Manage categories for specific user'),
        ),
        Str('memberuser_user?',
            label=_('Users'),
        ),
    )

@register()
class macmincat_add(LDAPCreate):
    __doc__ = _('Create new MAC min category')

    msg_summary = _('Added MAC min category"%(value)s"')

@register()
class macmincat_del(LDAPDelete):
    __doc__ = _('Delete MAC min category.')

    msg_summary = _('Deleted MAC min category"%(value)s"')

    def pre_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)
        return maccat_check( ldap, dn, keys)

@register()
class macmincat_mod(LDAPUpdate):
    __doc__ = _('Modify MAC min category.')

    msg_summary = _('Modified MAC min category"%(value)s"')

@register()
class macmincat_find(LDAPSearch):
    __doc__ = _('Search for MAC min category.')

    msg_summary = ngettext(
        '%(count)d MAC min category matched',
        '%(count)d MAC category min matched', 0
    )

    def pre_callback(self, ldap, filters, attrs_list,
                     base_dn, scope, *args, **options):
        assert isinstance(base_dn, DN)
        maccat_sync( ldap)
        return (filters, base_dn, scope)

@register()
class macmincat_show(LDAPRetrieve):
    __doc__ = _('Display MAC min category.')

@register()
class macmincat_add_macmincat(LDAPAddMember):

    __doc__ = _('Modify user min categories.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        try:
            user = options.get('user', [])
            maxcat_cn = api.Command.macmaxcat_show(keys[0])['result']
            members_maxcat = maxcat_cn.get('member_user')

            if members_maxcat == None or list(user)[0] not in members_maxcat:

                cn = keys[-1]
                attr = {'user':user[0]}

                api.Command.macmaxcat_add_macmaxcat(cn, **attr)
        except:
            logging.exception('macmincat_add: ')

        try:
            add_macmincat_attrs(ldap, dn, *keys, **options)
        except:
            logging.exception('add_macmincat_attrs: ')

        return (completed, dn)

@register()
class macmincat_remove_macmincat(LDAPRemoveMember):
    __doc__ = _('Remove users min categories.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        mov_macmincat_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

#topic = 'macmaxcat'

@register()
class macmaxcat(LDAPObject):
    """
    Levels max categories
    """
    container_dn = DN(('cn', 'macmaxcat'),
                      ('cn', 'mac-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('macmaxcat')
    object_name_plural = _('macmaxcats')
    object_class = ['x-ald-mac-category-object', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-mac-category', 'member']
    search_attributes = ['cn', 'x-ald-mac-category', 'member']

    attribute_members = {
        'member': ['user']
    }

    label = _('Max category levels')
    label_singular = _('Max category levels')

    takes_params = (
        Str('cn',
            cli_name='macmaxcat',
            label=_('Max category name'),
            primary_key=True,
        ),
        Str('x-ald-mac-category',
            cli_name='pcategoryid',
            label=_('max category rank'),
            doc=_('integer to order the MAC categories'),
        ),
        Str('uid?',
            cli_name='user',
            label=_('User name'),
            doc=_('Manage categories for specific user'),
        ),
        Str('memberuser_user?',
            label=_('Users'),
        ),
    )

@register()
class macmaxcat_add(LDAPCreate):
    __doc__ = _('Create new MAC max category')

    msg_summary = _('Added MAC max category"%(value)s"')

    logging.debug('PARSEC: macmaxcat_add');

@register()
class macmaxcat_del(LDAPDelete):
    __doc__ = _('Delete MAC max category.')

    msg_summary = _('Deleted MAC max category"%(value)s"')

    def pre_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)
        return maccat_check( ldap, dn, keys)

@register()
class macmaxcat_mod(LDAPUpdate):
    __doc__ = _('Modify MAC max category.')

    msg_summary = _('Modified MAC max category"%(value)s"')

@register()
class macmaxcat_find(LDAPSearch):
    __doc__ = _('Search for MAC max category.')

    msg_summary = ngettext(
        '%(count)d MAC max category matched',
        '%(count)d MAC category max matched', 0
    )
    logging.debug('PARSEC: macmaxcat_find');

    def pre_callback(self, ldap, filters, attrs_list,
                     base_dn, scope, *args, **options):
        assert isinstance(base_dn, DN)
        maccat_sync( ldap)
        return (filters, base_dn, scope)

@register()
class macmaxcat_show(LDAPRetrieve):
    __doc__ = _('Display MAC max category.')
    logging.debug('PARSEC: macmaxcat_show');

@register()
class macmaxcat_add_macmaxcat(LDAPAddMember):

    __doc__ = _('Modify user categories.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)
        logging.debug('PARSEC: macmaxcat_add_macmaxcat');

        add_macmaxcat_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class macmaxcat_remove_macmaxcat(LDAPRemoveMember):
    __doc__ = _('Remove users categories.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        try:
            user = options.get('user', [])
            mincat_cn = api.Command.macmincat_show(keys[0])['result']
            members_mincat = mincat_cn.get('member_user')

            if list(user)[0] in members_mincat:

                cn = keys[-1]
                attr = {'user':user[0]}

                api.Command.macmincat_remove_macmincat(cn, **attr)
        except:
            logging.exception('macmincat_remove_macmincat: ')

        try:
            mov_pcat_attrs(ldap, dn, *keys, **options)
        except:
            logging.exception('mov_pcat_attrs: ')

        return (completed, dn)

@register()
class macdevcat(LDAPObject):
    """
    Devices categories
    """
    container_dn = DN(('cn', 'macdevcat'),
                      ('cn', 'mac-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('macdevcat')
    object_name_plural = _('macdevcats')
    object_class = ['x-ald-mac-category-object', 'nestedgroup', 'groupofnames', 'nsContainer']
    default_attributes = ['cn', 'x-ald-mac-category', 'member']
    search_attributes = ['cn', 'x-ald-mac-category', 'member']

    attribute_members = {
        'member': ['parsecdevice']
    }

    label = _('Device category levels')
    label_singular = _('Device category levels')

    takes_params = (
        Str('cn',
            cli_name='devcat',
            label=_('Device category name'),
            primary_key=True,
        ),
        Str('x-ald-mac-category',
            cli_name='pcategoryid',
            label=_('Device category rank'),
            doc=_('integer to order the MAC categories'),
        ),
        Str('parsecdevice_device?',
            label=_('Devices'),
        ),
    )

@register()
class macdevcat_add(LDAPCreate):
    __doc__ = _('Create new MAC max category')

    msg_summary = _('Added MAC max category"%(value)s"')

    logging.debug('PARSEC: macmaxcat_add');

@register()
class macdevcat_del(LDAPDelete):
    __doc__ = _('Delete MAC max category.')

    msg_summary = _('Deleted MAC max category"%(value)s"')

@register()
class macdevcat_mod(LDAPUpdate):
    __doc__ = _('Modify MAC max category.')

    msg_summary = _('Modified MAC max category"%(value)s"')

@register()
class macdevcat_find(LDAPSearch):
    __doc__ = _('Search for MAC max category.')

    msg_summary = ngettext(
        '%(count)d MAC max category matched',
        '%(count)d MAC category max matched', 0
    )
    logging.debug('PARSEC: macmaxcat_find');

    def pre_callback(self, ldap, filters, attrs_list,
                     base_dn, scope, *args, **options):
        assert isinstance(base_dn, DN)
        maccat_sync( ldap)
        return (filters, base_dn, scope)

@register()
class macdevcat_show(LDAPRetrieve):
    __doc__ = _('Display MAC max category.')
    logging.debug('PARSEC: devcat_show');



@register()
class macdevcat_add_macdevcat(LDAPAddMember):

    __doc__ = _('Modify device categories.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        add_devicecat_attrs(ldap, dn, *keys, **options)

        return (completed, dn)

@register()
class macdevcat_remove_macdevcat(LDAPRemoveMember):
    __doc__ = _('Remove device categories.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        try:
            mov_device_attrs(ldap, dn, *keys, **options)
        except:
            logging.exception('mov_pcat_attrs: ')

        return (completed, dn)
